/*
 * Copyright (C) 2010 TechNexion Ltd.
 * Author: Hardy Weng <hardy.weng@technexion.com>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as  published by the
 * Free Software Foundation version 2.
 *
 * This program is distributed "as is" WITHOUT ANY WARRANTY of any kind,
 * whether express or implied; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */

#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/platform_device.h>
#include <linux/gpio.h>
#include <linux/irq.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/partitions.h>
#include <linux/mtd/nand.h>
#include <linux/input.h>
#include <linux/davinci_emac.h>
#include <linux/regulator/machine.h>
#include <linux/can/platform/ti_hecc.h>
#include <linux/interrupt.h>
#include <linux/mmc/host.h>
#include <linux/gpio_keys.h>

#include <mach/hardware.h>
#include <mach/am35xx.h>
#include <asm/mach-types.h>
#include <asm/mach/arch.h>
#include <asm/mach/map.h>

#include <plat/board.h>
#include <plat/common.h>
#include <plat/control.h>
#include <plat/usb.h>
#include <plat/display.h>
#include <plat/gpmc.h>
#include <plat/nand.h>
#include <plat/dma.h>
#include <plat/led.h>

#include <linux/spi/spi.h>
#include <linux/spi/ads7846.h>
#include <plat/mcspi.h>

#include <linux/smsc911x.h>
#include <linux/delay.h>

#include "mmc-tam3517.h"
#include "mux.h"

#include <linux/serial_8250.h>

#if defined(CONFIG_SERIAL_8520_XR16L2571) || defined(CONFIG_SERIAL_8520_XR16L2571_MODULE)

#define CONFIG_XR16L2571_PORTA_CS	1
#define CONFIG_XR16L2571_PORTB_CS       3
#define CONFIG_XR16L2571_PORTA_IRQ_PIN	155
#define CONFIG_XR16L2571_PORTB_IRQ_PIN	137
#define PORT(_base,_irq)                                \
        {                                               \
		.mapbase        = _base,		\
                .irq            = _irq,                 \
                .uartclk        = 14745600,            \
                .iotype         = UPIO_MEM,            \
                .flags          = UPF_BOOT_AUTOCONF|UPF_IOREMAP|UPF_SHARE_IRQ, \
		.regshift       = 1,	\
		.irqflags       = IRQF_SHARED | IRQF_TRIGGER_HIGH, \
        }

static struct plat_serial8250_port xr16l2571_ports[] = {
	PORT(0x21000000,OMAP_GPIO_IRQ(CONFIG_XR16L2571_PORTA_IRQ_PIN)),
	PORT(0x23000000,OMAP_GPIO_IRQ(CONFIG_XR16L2571_PORTB_IRQ_PIN)),
	{}
};

static struct platform_device xr16l2571_device =
{
        .name                   = "serial8250",
        .id                     = 3,
        .dev                    = {
                .platform_data  = &xr16l2571_ports,
	},
};


static inline void __init tam3517_init_xr16l2571(void)
{
        unsigned long cs_mem_base;

        if (gpmc_cs_request(CONFIG_XR16L2571_PORTA_CS, SZ_16M, &cs_mem_base) < 0) {
                printk(KERN_ERR "Failed request for GPMC mem for xr16l2571 porta\n");
                return;
        }
	
	if(gpio_request(CONFIG_XR16L2571_PORTA_IRQ_PIN, "xr16l2571 porta irq"))
	{
		printk(KERN_ERR "Failed request gpio for xr16l2571 porta irq\n");
		return;
	}
        omap_mux_init_gpio(CONFIG_XR16L2571_PORTA_IRQ_PIN, OMAP_PIN_INPUT|OMAP_MUX_MODE4);

        gpio_direction_input(CONFIG_XR16L2571_PORTA_IRQ_PIN);

	if (gpmc_cs_request(CONFIG_XR16L2571_PORTB_CS, SZ_16M, &cs_mem_base) < 0) {
                printk(KERN_ERR "Failed request for GPMC mem for xr16l2571 portb\n");
                return;
        }

        if(gpio_request(CONFIG_XR16L2571_PORTB_IRQ_PIN, "xr16l2571 portb irq"))
	{
		printk(KERN_ERR "Failed request gpio for xr16l2571 portb irq\n");
		return;
	}
        omap_mux_init_gpio(CONFIG_XR16L2571_PORTB_IRQ_PIN, OMAP_PIN_INPUT|OMAP_MUX_MODE4);

        gpio_direction_input(CONFIG_XR16L2571_PORTB_IRQ_PIN);

}
#endif


#if defined(CONFIG_MTD_NAND_OMAP2) || defined(CONFIG_MTD_NAND_OMAP2_MODULE)
#define GPMC_CS0_BASE  0x60
#define GPMC_CS_SIZE   0x30

#define NAND_BLOCK_SIZE        SZ_128K


static struct mtd_partition tam3517_nand_partitions[] = {
{
       .name           = "xloader",
       .offset         = 0,
       .size           = 4*NAND_BLOCK_SIZE,
       .mask_flags     = MTD_WRITEABLE
},
{
       .name           = "uboot",
       .offset         = MTDPART_OFS_APPEND,
       .size           = 0x100000,
       .mask_flags     = MTD_WRITEABLE
},
{
       .name           = "params",
       .offset         = MTDPART_OFS_APPEND,
       .size           = 1*(NAND_BLOCK_SIZE)
},
{
       .name           = "kernel",
       .offset         = MTDPART_OFS_APPEND,
       .size           = 0x600000
},
{
       .name           = "rootfs",
       .size           = MTDPART_SIZ_FULL,
       .offset         = MTDPART_OFS_APPEND,
},
};

static struct omap_nand_platform_data tam3517_nand_data = {
       .parts          = tam3517_nand_partitions,
       .nr_parts       = ARRAY_SIZE(tam3517_nand_partitions),
       .nand_setup     = NULL,
       .dma_channel    = -1,           /* disable DMA in OMAP NAND driver */
       .dev_ready      = NULL,
};

static struct resource tam3517_nand_resource = {
       .flags          = IORESOURCE_MEM,
};

static struct platform_device tam3517_nand_device = {
       .name           = "omap2-nand",
       .id             = 0,
       .dev            = {
                       .platform_data  = &tam3517_nand_data,
       },
       .num_resources  = 1,
       .resource       = &tam3517_nand_resource,
};

void __init tam3517_flash_init(void)
{
       u8 cs = 0;
       u8 nandcs = GPMC_CS_NUM + 1;
       u32 gpmc_base_add = OMAP34XX_GPMC_VIRT;

       while (cs < GPMC_CS_NUM) {
               u32 ret = 0;
               ret = gpmc_cs_read_reg(cs, GPMC_CS_CONFIG1);

               if ((ret & 0xC00) == 0x800) {
                       /* Found it!! */
                       if (nandcs > GPMC_CS_NUM)
                               nandcs = cs;
               }
               cs++;
       }
       if (nandcs > GPMC_CS_NUM) {
               printk(KERN_INFO "NAND: Unable to find configuration "
                       " in GPMC\n ");
               return;
       }

       if (nandcs < GPMC_CS_NUM) {
               tam3517_nand_data.cs   = nandcs;
               tam3517_nand_data.gpmc_cs_baseaddr = (void *)(gpmc_base_add +
                                       GPMC_CS0_BASE + nandcs*GPMC_CS_SIZE);
               tam3517_nand_data.gpmc_baseaddr   = (void *) (gpmc_base_add);

               if (platform_device_register(&tam3517_nand_device) < 0)
                       printk(KERN_ERR "Unable to register NAND device\n");

       }
}
#else
void __init tam3517_flash_init(void) {}
#endif

#if defined(CONFIG_SMSC911X) || defined(CONFIG_SMSC911X_MODULE)

#if !defined(CONFIG_TAM3517_SMSC911X_IRQ_PIN)
#define CONFIG_TAM3517_SMSC911X_IRQ_PIN  153
#endif
#if !defined(CONFIG_TAM3517_SMSC911X_CS)
#define CONFIG_TAM3517_SMSC911X_CS    5
#endif
#if !defined(CONFIG_TAM3517_SMSC911X_RESET_PIN)
#define	CONFIG_TAM3517_SMSC911X_RESET_PIN	142
#endif
static struct resource tam3517_smsc911x_resources[] = {
        [0] =   {
                .flags  = IORESOURCE_MEM,
        },
        [1] =   {
                .start  = OMAP_GPIO_IRQ(CONFIG_TAM3517_SMSC911X_IRQ_PIN),
                .end    = 0,
                .flags  = (IORESOURCE_IRQ | IORESOURCE_IRQ_LOWLEVEL ),
        },
};

static struct smsc911x_platform_config smsc911x_config = {
        .phy_interface  = PHY_INTERFACE_MODE_MII,
        .irq_polarity   = SMSC911X_IRQ_POLARITY_ACTIVE_LOW,
        .irq_type       = SMSC911X_IRQ_TYPE_OPEN_DRAIN,
        .flags          = SMSC911X_USE_16BIT,
};

static struct platform_device tam3517_smsc911x_device = {
        .name           = "smsc911x",
        .id             = 1,
        .num_resources  = ARRAY_SIZE(tam3517_smsc911x_resources),
        .resource       = &tam3517_smsc911x_resources[0],
        .dev            = {
                .platform_data = &smsc911x_config,
        },
};
static inline void __init tam3517_init_smsc911x(void)
{
        unsigned long cs_mem_base;

	if (gpmc_cs_request(CONFIG_TAM3517_SMSC911X_CS, SZ_16M, &cs_mem_base) < 0) {
                printk(KERN_ERR "Failed request for GPMC mem for smsc911x\n");
                return;
        }

        tam3517_smsc911x_resources[0].start = cs_mem_base + 0x0;
        tam3517_smsc911x_resources[0].end   = cs_mem_base + 0xff;

        if( gpio_request(CONFIG_TAM3517_SMSC911X_IRQ_PIN, "smsc911x irq") < 0) {
                printk(KERN_ERR "can't get smsc911x irq GPIO\n");
                return;
        }

        omap_mux_init_gpio(CONFIG_TAM3517_SMSC911X_IRQ_PIN, OMAP_PIN_INPUT_PULLUP|OMAP_MUX_MODE4);

        gpio_direction_input(CONFIG_TAM3517_SMSC911X_IRQ_PIN);
}

static int __init smsc911x_eth_addr_setup(char *str)
{
        int i;

        if(str == NULL)
                return 0;
        for(i = 0; i <  ETH_ALEN; i++)
                smsc911x_config.mac[i] = simple_strtol(&str[i*3],
                                                        (char **)NULL, 16);
        return 1;
}

__setup("eth1addr=", smsc911x_eth_addr_setup);

#else
static inline void __init tam3517_init_smsc911x(void) { return; }
#endif

#if defined(CONFIG_TI_DAVINCI_EMAC) || defined(CONFIG_TI_DAVINCI_EMAC_MODULE)

#define AM35XX_EVM_PHY_MASK		(0xF)
#define AM35XX_EVM_MDIO_FREQUENCY    	(1000000)

static struct emac_platform_data tam3517_emac_pdata = {
	.phy_mask       = AM35XX_EVM_PHY_MASK,
	.mdio_max_freq  = AM35XX_EVM_MDIO_FREQUENCY,
	.rmii_en        = 1,
};

static int __init emac_eth_addr_setup(char *str)
{
	int i;

	if(str == NULL)
		return 0;
	for(i = 0; i <  ETH_ALEN; i++)
		tam3517_emac_pdata.mac_addr[i] = simple_strtol(&str[i*3],
							(char **)NULL, 16);
	return 1;
}

__setup("eth0addr=", emac_eth_addr_setup);

static struct resource am3517_emac_resources[] = {
	{
		.start  = AM35XX_IPSS_EMAC_BASE,
		.end    = AM35XX_IPSS_EMAC_BASE + 0x3FFFF,
		.flags  = IORESOURCE_MEM,
	},
	{
		.start  = INT_35XX_EMAC_C0_RXTHRESH_IRQ,
		.end    = INT_35XX_EMAC_C0_RXTHRESH_IRQ,
		.flags  = IORESOURCE_IRQ,
	},
	{
		.start  = INT_35XX_EMAC_C0_RX_PULSE_IRQ,
		.end    = INT_35XX_EMAC_C0_RX_PULSE_IRQ,
		.flags  = IORESOURCE_IRQ,
	},
	{
		.start  = INT_35XX_EMAC_C0_TX_PULSE_IRQ,
		.end    = INT_35XX_EMAC_C0_TX_PULSE_IRQ,
		.flags  = IORESOURCE_IRQ,
	},
	{
		.start  = INT_35XX_EMAC_C0_MISC_PULSE_IRQ,
		.end    = INT_35XX_EMAC_C0_MISC_PULSE_IRQ,
		.flags  = IORESOURCE_IRQ,
	},
};

static struct platform_device am3517_emac_device = {
	.name           = "davinci_emac",
	.id             = -1,
	.num_resources  = ARRAY_SIZE(am3517_emac_resources),
	.resource       = am3517_emac_resources,
};

static void am3517_enable_ethernet_int(void)
{
	u32 regval;

	regval = omap_ctrl_readl(AM35XX_CONTROL_LVL_INTR_CLEAR);
	regval = (regval | AM35XX_CPGMAC_C0_RX_PULSE_CLR |
		AM35XX_CPGMAC_C0_TX_PULSE_CLR | AM35XX_CPGMAC_C0_MISC_PULSE_CLR |
		AM35XX_CPGMAC_C0_RX_THRESH_CLR );
	omap_ctrl_writel(regval,AM35XX_CONTROL_LVL_INTR_CLEAR);
	regval = omap_ctrl_readl(AM35XX_CONTROL_LVL_INTR_CLEAR);
}

static void am3517_disable_ethernet_int(void)
{
	u32 regval;

	regval = omap_ctrl_readl(AM35XX_CONTROL_LVL_INTR_CLEAR);
	regval = (regval | AM35XX_CPGMAC_C0_RX_PULSE_CLR |
		AM35XX_CPGMAC_C0_TX_PULSE_CLR);
	omap_ctrl_writel(regval,AM35XX_CONTROL_LVL_INTR_CLEAR);
	regval = omap_ctrl_readl(AM35XX_CONTROL_LVL_INTR_CLEAR);
}

static void inline tam3517_ethernet_init(struct emac_platform_data *pdata)
{
	unsigned int regval;

	pdata->ctrl_reg_offset          = AM35XX_EMAC_CNTRL_OFFSET;
	pdata->ctrl_mod_reg_offset      = AM35XX_EMAC_CNTRL_MOD_OFFSET;
	pdata->ctrl_ram_offset          = AM35XX_EMAC_CNTRL_RAM_OFFSET;
	pdata->mdio_reg_offset          = AM35XX_EMAC_MDIO_OFFSET;
	pdata->ctrl_ram_size            = AM35XX_EMAC_CNTRL_RAM_SIZE;
	pdata->version                  = EMAC_VERSION_2;
	pdata->hw_ram_addr              = AM35XX_EMAC_HW_RAM_ADDR;
	pdata->interrupt_enable 	= am3517_enable_ethernet_int;
	pdata->interrupt_disable 	= am3517_disable_ethernet_int;
	am3517_emac_device.dev.platform_data     = pdata;

	regval = omap_ctrl_readl(AM35XX_CONTROL_IP_SW_RESET);
	regval = regval & (~(AM35XX_CPGMACSS_SW_RST));
	omap_ctrl_writel(regval,AM35XX_CONTROL_IP_SW_RESET);
	regval = omap_ctrl_readl(AM35XX_CONTROL_IP_SW_RESET);

	return ;
}
#else
static void inline tam3517_ethernet_init(struct emac_platform_data *pdata) {}
#endif

#if !defined(CONFIG_TAM3517_TS_IRQ_PIN)
#define CONFIG_TAM3517_TS_IRQ_PIN	136
#endif

#if defined(CONFIG_TOUCHSCREEN_ADS7846) || defined(CONFIG_TOUCHSCREEN_ADS7846_MODULE)

static int ads7846_get_pendown_state(void)
{
        return !gpio_get_value(CONFIG_TAM3517_TS_IRQ_PIN);
}

static struct ads7846_platform_data tsc2046_config __initdata = {
	.x_max                  = 0x0fff,
        .y_max                  = 0x0fff,
        .x_plate_ohms           = 80,
        .pressure_max           = 255,
        .debounce_max           = 30,
        .debounce_tol           = 10,
        .debounce_rep           = 1,
        .get_pendown_state      = ads7846_get_pendown_state,
        .keep_vref_on           = 1,
        .settle_delay_usecs     = 100,
};

static struct omap2_mcspi_device_config tsc2046_mcspi_config = {
        .turbo_mode     = 0,
        .single_channel = 1,    /* 0: slave, 1: master */
};

static struct spi_board_info tam3517_spi_ts_info[] __initdata = {
        [0] = {
                /*
                 * TSC2046 operates at a max freqency of 2MHz, so
                 * operate slightly below at 1.5MHz
                 */
                .modalias               = "ads7846",
                .bus_num                = 1,
                .chip_select            = 0,
                .max_speed_hz           = 1500000,
                .controller_data        = &tsc2046_mcspi_config,
                .irq                    = OMAP_GPIO_IRQ(CONFIG_TAM3517_TS_IRQ_PIN),
                .platform_data          = &tsc2046_config,
        },
};
static void inline ads7846_dev_init(void)
{
        if (gpio_request(CONFIG_TAM3517_TS_IRQ_PIN, "ADS7846 pendown") < 0) {
                printk(KERN_ERR "can't get ads746 pen down GPIO\n");
                return;
        }
        omap_mux_init_gpio(CONFIG_TAM3517_TS_IRQ_PIN, OMAP_PIN_INPUT_PULLUP|OMAP_MUX_MODE4);

        gpio_direction_input(CONFIG_TAM3517_TS_IRQ_PIN);

        omap_set_gpio_debounce(CONFIG_TAM3517_TS_IRQ_PIN, 1);
        omap_set_gpio_debounce_time(CONFIG_TAM3517_TS_IRQ_PIN, 0xa);

         spi_register_board_info(tam3517_spi_ts_info,
                                ARRAY_SIZE(tam3517_spi_ts_info));
}
#else
static void inline ads7846_dev_init(void) {}
#endif

#if defined(CONFIG_FB_OMAP2) || defined(CONFIG_FB_OMAP2_MODULE)

#if !defined(CONFIG_PANEL_PWR_PIN)
#define CONFIG_PANEL_PWR_PIN		138
#endif
#if !defined(CONFIG_LCD_PANEL_BKLIGHT_PWR_PIN)
#define CONFIG_LCD_PANEL_BKLIGHT_PWR_PIN	53
#endif
#if !defined(CONFIG_LCD_PANEL_PON_PIN)
#define CONFIG_LCD_PANEL_PON_PIN		139
#endif
#if !defined(CONFIG_DVI_PON_PIN)
#define	CONFIG_DVI_PON_PIN		24
#endif


static void tam3517_bklight_set_power(struct omap_pwm_led_platform_data *self, int on_off)
{

}
static struct omap_pwm_led_platform_data tam3517_bklight = {
        .name   =       "lcd-backlight",
        .intensity_timer        =       9,
        .blink_timer            =       0,
        .set_power              =       tam3517_bklight_set_power,
};

struct platform_device tam3517_bklight_device = {
        .name           = "omap_pwm_led",
        .id             = 1,
        .dev            = {
                .platform_data  = &tam3517_bklight,
        },
};

static int tam3517_panel_enable_lcd(struct omap_dss_device *dssdev)
{
	gpio_set_value(CONFIG_DVI_PON_PIN,0);
	gpio_set_value(CONFIG_PANEL_PWR_PIN, 0);
	gpio_set_value(CONFIG_LCD_PANEL_PON_PIN, 1);

	return 0;
}

static void tam3517_panel_disable_lcd(struct omap_dss_device *dssdev)
{
	gpio_set_value(CONFIG_PANEL_PWR_PIN, 1);
	gpio_set_value(CONFIG_LCD_PANEL_PON_PIN, 0);
}


static struct omap_dss_device tam3517_lcd_device = {
	.type                   = OMAP_DISPLAY_TYPE_DPI,
        .name                   = "lcd",
        .driver_name            = "generic_panel",
        .platform_enable        = tam3517_panel_enable_lcd,
        .platform_disable       = tam3517_panel_disable_lcd,

//        .get_backlight          = tam3517_get_backlight,
//        .set_backlight          = tam3517_set_backlight,
        .max_backlight_level    = 100,

};

static int tam3517_panel_enable_tv(struct omap_dss_device *dssdev)
{
	return 0;
}

static void tam3517_panel_disable_tv(struct omap_dss_device *dssdev)
{
}

static struct omap_dss_device tam3517_tv_device = {
	.type 			= OMAP_DISPLAY_TYPE_VENC,
	.name 			= "tv",
	.driver_name		= "venc",
	.phy.venc.type		= OMAP_DSS_VENC_TYPE_SVIDEO,
	.platform_enable	= tam3517_panel_enable_tv,
	.platform_disable	= tam3517_panel_disable_tv,
};

static int tam3517_panel_enable_dvi(struct omap_dss_device *dssdev)
{
	gpio_set_value(CONFIG_PANEL_PWR_PIN, 1);
	gpio_set_value(CONFIG_LCD_PANEL_PON_PIN, 0);
	gpio_set_value(CONFIG_DVI_PON_PIN,1);
        return 0;
}

static void tam3517_panel_disable_dvi(struct omap_dss_device *dssdev)
{
        gpio_set_value(CONFIG_DVI_PON_PIN,0);
}

static struct omap_dss_device tam3517_dvi_device = {
        .type                   = OMAP_DISPLAY_TYPE_DPI,
        .name                   = "dvi",
        .driver_name            = "generic_panel",
        .platform_enable        = tam3517_panel_enable_dvi,
        .platform_disable       = tam3517_panel_disable_dvi,
};

static struct omap_dss_device *tam3517_dss_devices[] = {
	&tam3517_tv_device,
	&tam3517_lcd_device,
	&tam3517_dvi_device,
};

static struct omap_dss_board_info tam3517_dss_data = {
	.num_devices	= ARRAY_SIZE(tam3517_dss_devices),
	.devices	= tam3517_dss_devices,
	.default_device	= &tam3517_lcd_device,
};

struct platform_device tam3517_dss_device = {
	.name		= "omapdss",
	.id		= -1,
	.dev		= {
		.platform_data	= &tam3517_dss_data,
	},
};

static void __init tam3517_display_init(void)
{
        int r;
        omap_mux_init_gpio(CONFIG_PANEL_PWR_PIN, OMAP_PIN_INPUT_PULLUP);
        omap_mux_init_gpio(CONFIG_LCD_PANEL_PON_PIN, OMAP_PIN_INPUT_PULLDOWN);
        omap_mux_init_gpio(CONFIG_DVI_PON_PIN, OMAP_PIN_INPUT_PULLUP);
        /*
         * Enable GPIO = Panel Power enable pin
         */
        r = gpio_request(CONFIG_PANEL_PWR_PIN, "panel_pwr");
        if (r) {
                printk(KERN_ERR "failed to get panel_pwr\n");
                return;
        }
        gpio_direction_output(CONFIG_PANEL_PWR_PIN, 1);
        /*
         * Enable GPIO = LCD Panel enable pin
         */
        r = gpio_request(CONFIG_LCD_PANEL_PON_PIN, "lcd_panel_pon");
        if (r) {
                printk(KERN_ERR "failed to get lcd_panel_pon\n");
                goto err_1;
        }
        gpio_direction_output(CONFIG_LCD_PANEL_PON_PIN, 0);

        /*
         * Enable GPIO = LCD Panel enable pin
         */
        r = gpio_request(CONFIG_DVI_PON_PIN, "dvi_pon");
        if (r) {
                printk(KERN_ERR "failed to get dvi_pon\n");
                goto err_2;
        }
        gpio_direction_output(CONFIG_DVI_PON_PIN, 0);

        printk(KERN_INFO "Display initialized successfully\n");
        return;

err_2:
	gpio_free(CONFIG_LCD_PANEL_PON_PIN);
err_1:
        gpio_free(CONFIG_PANEL_PWR_PIN);

}

#else
static void __init tam3517_display_init(void) {}
#endif


/* TPS65023 specific initialization */
/* VDCDC1 -> VDD_CORE */
static struct regulator_consumer_supply tam3517_vdcdc1_supplies[] = {
	{
		.supply = "vdd_core",
	},
};

/* VDCDC2 -> VDDSHV */
static struct regulator_consumer_supply tam3517_vdcdc2_supplies[] = {
	{
		.supply = "vddshv",
	},
};

/* VDCDC2 |-> VDDS
	   |-> VDDS_SRAM_CORE_BG
	   |-> VDDS_SRAM_MPU */
static struct regulator_consumer_supply tam3517_vdcdc3_supplies[] = {
	{
		.supply = "vdds",
	},
	{
		.supply = "vdds_sram_core_bg",
	},
	{
		.supply = "vdds_sram_mpu",
	},
};

/* LDO1 |-> VDDA1P8V_USBPHY
	 |-> VDDA_DAC */
static struct regulator_consumer_supply tam3517_ldo1_supplies[] = {
	{
		.supply = "vdda1p8v_usbphy",
	},
	{
		.supply = "vdda_dac",
	},
};

/* LDO2 -> VDDA3P3V_USBPHY */
static struct regulator_consumer_supply tam3517_ldo2_supplies[] = {
	{
		.supply = "vdda3p3v_usbphy",
	},
};

static struct regulator_init_data tam3517_regulator_data[] = {
	/* DCDC1 */
	{
		.constraints = {
			.min_uV = 1200000,
			.max_uV = 1200000,
			.valid_modes_mask = REGULATOR_MODE_NORMAL,
			.valid_ops_mask = REGULATOR_CHANGE_STATUS,
			.always_on = true,
			.apply_uV = false,
		},
		.num_consumer_supplies = ARRAY_SIZE(tam3517_vdcdc1_supplies),
		.consumer_supplies = tam3517_vdcdc1_supplies,
	},
	/* DCDC2 */
	{
		.constraints = {
			.min_uV = 3300000,
			.max_uV = 3300000,
			.valid_modes_mask = REGULATOR_MODE_NORMAL,
			.valid_ops_mask = REGULATOR_CHANGE_STATUS,
			.always_on = true,
			.apply_uV = false,
		},
		.num_consumer_supplies = ARRAY_SIZE(tam3517_vdcdc2_supplies),
		.consumer_supplies = tam3517_vdcdc2_supplies,
	},
	/* DCDC3 */
	{
		.constraints = {
			.min_uV = 1800000,
			.max_uV = 1800000,
			.valid_modes_mask = REGULATOR_MODE_NORMAL,
			.valid_ops_mask = REGULATOR_CHANGE_STATUS,
			.always_on = true,
			.apply_uV = false,
		},
		.num_consumer_supplies = ARRAY_SIZE(tam3517_vdcdc3_supplies),
		.consumer_supplies = tam3517_vdcdc3_supplies,
	},
	/* LDO1 */
	{
		.constraints = {
			.min_uV = 1800000,
			.max_uV = 1800000,
			.valid_modes_mask = REGULATOR_MODE_NORMAL,
			.valid_ops_mask = REGULATOR_CHANGE_STATUS,
			.always_on = false,
			.apply_uV = false,
		},
		.num_consumer_supplies = ARRAY_SIZE(tam3517_ldo1_supplies),
		.consumer_supplies = tam3517_ldo1_supplies,
	},
	/* LDO2 */
	{
		.constraints = {
			.min_uV = 3300000,
			.max_uV = 3300000,
			.valid_modes_mask = REGULATOR_MODE_NORMAL,
			.valid_ops_mask = REGULATOR_CHANGE_STATUS,
			.always_on = false,
			.apply_uV = false,
		},
		.num_consumer_supplies = ARRAY_SIZE(tam3517_ldo2_supplies),
		.consumer_supplies = tam3517_ldo2_supplies,
	},
};


static struct i2c_board_info __initdata tam3517_i2c1_boardinfo[] = {
        {
                I2C_BOARD_INFO("tps65023", 0x48),
                .flags = I2C_CLIENT_WAKE,
                .platform_data = &tam3517_regulator_data[0],
        },
        {
                I2C_BOARD_INFO("24c02", 0x50),
        },
        {
                I2C_BOARD_INFO("tlv320aic23", 0x1a),
        },
};

static struct i2c_board_info __initdata tam3517_i2c2_boardinfo[] = {
        {
                I2C_BOARD_INFO("24c02", 0x50),
        },
};

static struct i2c_board_info __initdata tam3517_i2c3_boardinfo[] = {
        {
                I2C_BOARD_INFO("s35390a", 0x30),
                .type           = "s35390a",
        },
        {
                I2C_BOARD_INFO("ds1307", 0x68),
        },
};


static int __init tam3517_i2c_init(void)
{
	omap_register_i2c_bus(1, 400, tam3517_i2c1_boardinfo,
			ARRAY_SIZE(tam3517_i2c1_boardinfo));
	omap_register_i2c_bus(2, 400, tam3517_i2c2_boardinfo,
			ARRAY_SIZE(tam3517_i2c2_boardinfo));
	omap_register_i2c_bus(3, 400, tam3517_i2c3_boardinfo,
                        ARRAY_SIZE(tam3517_i2c3_boardinfo));
	return 0;
}

#if defined(CONFIG_CAN_TI_HECC) || defined(CONFIG_CAN_TI_HECC_MODULE)
/*
 * HECC information
 */

static void am3517_hecc_plat_init(void)
{
}

static struct resource am3517_hecc_resources[] = {
        {
                .start  = AM35XX_IPSS_HECC_BASE,
                .end    = AM35XX_IPSS_HECC_BASE + 0x3FFF,
                .flags  = IORESOURCE_MEM,
        },
        {
                .start  = INT_35XX_HECC0_IRQ,
                .end    = INT_35XX_HECC0_IRQ,
                .flags  = IORESOURCE_IRQ,
        },
};

static struct ti_hecc_platform_data tam3517_hecc_pdata = {
        .scc_hecc_offset        = AM35XX_HECC_SCC_HECC_OFFSET,
        .scc_ram_offset         = AM35XX_HECC_SCC_RAM_OFFSET,
        .hecc_ram_offset        = AM35XX_HECC_RAM_OFFSET,
        .mbx_offset            = AM35XX_HECC_MBOX_OFFSET,
        .int_line               = AM35XX_HECC_INT_LINE,
        .version                = AM35XX_HECC_VERSION,
	.platform_init		= am3517_hecc_plat_init,
};

static struct platform_device am3517_hecc_device = {
        .name           = "ti_hecc",
        .id             = 1,
        .num_resources  = ARRAY_SIZE(am3517_hecc_resources),
        .resource       = am3517_hecc_resources,
	.dev.platform_data = &tam3517_hecc_pdata,
};
#endif


/*
 * Board initialization
 */
static struct omap_board_config_kernel tam3517_config[] __initdata = {
};


#if defined(CONFIG_LEDS_OMAP) || defined(CONFIG_LEDS_OMAP_MODULE)

static struct omap_led_config tam3517_led_config[] = {
	{ .cdev.default_trigger = "heartbeat", 
	  .cdev.name = "status", 
	  .gpio = 57 },
};
static struct omap_led_platform_data tam3517_status_led = {
        .nr_leds        = ARRAY_SIZE(tam3517_led_config),
        .leds           = tam3517_led_config,
};

struct platform_device tam3517_status_led_device = {
        .name           = "omap-led",
        .id             = -1,
        .dev            = {
                .platform_data  = &tam3517_status_led,
        },
};
#endif

#if defined(CONFIG_KEYBOARD_GPIO) || defined(CONFIG_KEYBOARD_GPIO_MODULE)
static struct gpio_keys_button gpio_buttons[] = {
        {
                .code                   = KEY_HOME,
                .gpio                   = 65,
                .desc                   = "home",
                .wakeup                 = 1,
		.active_low		= 1,
        },
        {
                .code                   = KEY_ENTER,
                .gpio                   = 64,
                .desc                   = "enter",
                .wakeup                 = 1,
		.active_low		= 1,
        },
        {
                .code                   = KEY_BACK,
                .gpio                   = 63,
                .desc                   = "back",
                .wakeup                 = 1,
		.active_low		= 1,
        },
        {
                .code                   = KEY_MENU,
                .gpio                   = 150,
                .desc                   = "menu",
                .wakeup                 = 1,
		.active_low		= 1,
        },
        {
                .code                   = KEY_BRIGHTNESSUP,
                .gpio                   = 143,
                .desc                   = "brightness up",
                .wakeup                 = 1,
                .active_low             = 1,
        },
        {
                .code                   = KEY_BRIGHTNESSDOWN,
                .gpio                   = 142,
                .desc                   = "brightness down",
                .wakeup                 = 1,
                .active_low             = 1,
        },
        {
                .code                   = KEY_VOLUMEUP,
                .gpio                   = 141,
                .desc                   = "volume up",
                .wakeup                 = 1,
		.active_low		= 1,
        },
        {
                .code                   = KEY_VOLUMEDOWN,
                .gpio                   = 140,
                .desc                   = "volume down",
                .wakeup                 = 1,
		.active_low		= 1,
        },

};

static struct gpio_keys_platform_data gpio_key_info = {
        .buttons        = gpio_buttons,
        .nbuttons       = ARRAY_SIZE(gpio_buttons),
};

static struct platform_device keys_gpio = {
        .name   = "gpio-keys",
        .id     = -1,
        .dev    = {
                .platform_data  = &gpio_key_info,
        },
};
#endif

static struct platform_device *tam3517_devices[] __initdata = {
#if defined(CONFIG_FB_OMAP2) || defined(CONFIG_FB_OMAP2_MODULE)
	&tam3517_dss_device,
	&tam3517_bklight_device,
#endif
#if defined(CONFIG_LEDS_OMAP) || defined(CONFIG_LEDS_OMAP_MODULE)
	&tam3517_status_led_device,
#endif
#if defined(CONFIG_CAN_TI_HECC) || defined(CONFIG_CAN_TI_HECC_MODULE)
        &am3517_hecc_device,
#endif
#if defined(CONFIG_TI_DAVINCI_EMAC) || defined(CONFIG_TI_DAVINCI_EMAC_MODULE)
	&am3517_emac_device,
#endif
#if defined(CONFIG_SMSC911X) || defined(CONFIG_SMSC911X_MODULE)
	&tam3517_smsc911x_device,
#endif
#if defined(CONFIG_SERIAL_8520_XR16L2571)
	&xr16l2571_device,
#endif
#if defined(CONFIG_KEYBOARD_GPIO) || defined(CONFIG_KEYBOARD_GPIO_MODULE)
	&keys_gpio,
#endif
};

static void __init tam3517_init_irq(void)
{
	omap_board_config = tam3517_config;
	omap_board_config_size = ARRAY_SIZE(tam3517_config);

	omap2_init_common_hw(NULL, NULL, NULL, NULL, NULL);
	omap_init_irq();
	omap_gpio_init();
}

#if defined(CONFIG_USB_EHCI_HCD) || defined(CONFIG_USB_EHCI_HCD_MODULE)
#if !defined(CONFIG_TAM3517_EHCI_RESET_PIN)
#define CONFIG_TAM3517_EHCI_RESET_PIN	25
#endif
static struct ehci_hcd_omap_platform_data ehci_pdata __initdata = {
	.port_mode[0] = EHCI_HCD_OMAP_MODE_PHY,
	.port_mode[1] = EHCI_HCD_OMAP_MODE_UNKNOWN,
	.port_mode[2] = EHCI_HCD_OMAP_MODE_UNKNOWN,

	.phy_reset  = true,
	.reset_gpio_port[0]  = CONFIG_TAM3517_EHCI_RESET_PIN,
	.reset_gpio_port[1]  = -EINVAL,
	.reset_gpio_port[2]  = -EINVAL
};
#endif

#ifdef CONFIG_OMAP_MUX
static struct omap_board_mux board_mux[] __initdata = {
	/* USB OTG DRVVBUS offset = 0x212 */
	OMAP3_MUX(CHASSIS_DMAREQ3, OMAP_MUX_MODE0 | OMAP_PIN_INPUT_PULLDOWN),
	OMAP3_MUX(MCBSP_CLKS, OMAP_MUX_MODE4 | OMAP_PIN_INPUT_PULLUP),
	OMAP3_MUX(GPMC_NCS4, OMAP_MUX_MODE4 | OMAP_PIN_INPUT_PULLDOWN),
	OMAP3_MUX(GPMC_NCS6, OMAP_MUX_MODE4 ),
	OMAP3_MUX(GPMC_NCS7, OMAP_MUX_MODE4 | OMAP_PULL_UP),
	{ .reg_offset = OMAP_MUX_TERMINATOR },
};
#else
#define board_mux	NULL
#endif

#if !defined(CONFIG_TAM3517_MMC1_CD_PIN)
#define CONFIG_TAM3517_MMC1_CD_PIN	126
#endif
#if !defined(CONFIG_TAM3517_MMC1_WP_PIN)
#define CONFIG_TAM3517_MMC1_WP_PIN	-EINVAL
#endif
static struct tam3517_hsmmc_info mmc[] = {
	{
		.mmc            = 1,
		.wires          = 4,
		.gpio_cd        = CONFIG_TAM3517_MMC1_CD_PIN,
		.gpio_wp        = CONFIG_TAM3517_MMC1_WP_PIN,
		.ocr_mask       = MMC_VDD_165_195 |
                             MMC_VDD_26_27 | MMC_VDD_27_28 |
                             MMC_VDD_29_30 |
                             MMC_VDD_30_31 | MMC_VDD_31_32,
	},
        {
                .mmc            = 2,
                .wires          = 4,
                .gpio_cd        = -EINVAL,
                .gpio_wp        = -EINVAL,
                .nonremovable   = true,
                .ocr_mask       = MMC_VDD_165_195 |
                             MMC_VDD_26_27 | MMC_VDD_27_28 |
                             MMC_VDD_29_30 |
                             MMC_VDD_30_31 | MMC_VDD_31_32,
        },
	{}
};

#ifndef CONFIG_TAM3517_MMC2_POWER_PIN
#define CONFIG_TAM3517_MMC2_POWER_PIN   156
#endif

static void tam3517_mmc2_wifi_init(void)
{
        omap_mux_init_gpio(CONFIG_TAM3517_MMC2_POWER_PIN, OMAP_PIN_OUTPUT|OMAP_MUX_MODE4);

        gpio_direction_output(CONFIG_TAM3517_MMC2_POWER_PIN,1);
}

#ifndef CONFIG_TAM3517_POWER_RESET_PIN
#define CONFIG_TAM3517_POWER_RESET_PIN  31
#endif


static void __init tam3517_init(void)
{


	tam3517_i2c_init();

	omap3_mux_init(board_mux, OMAP_PACKAGE_CBB);

	omap_serial_init_port(0);
	omap_serial_init_port(1);
	omap_serial_init_port(2);

#if defined(CONFIG_SERIAL_8520_XR16L2571)
        tam3517_init_xr16l2571();
#endif
#if defined(CONFIG_SMSC911X) || defined(CONFIG_SMSC911X_MODULE)
	tam3517_init_smsc911x();
#endif
#if defined(CONFIG_TI_DAVINCI_EMAC) || defined(CONFIG_TI_DAVINCI_EMAC_MODULE)
	tam3517_ethernet_init(&tam3517_emac_pdata);
#endif
	tam3517_display_init();

	platform_add_devices(tam3517_devices,
                                ARRAY_SIZE(tam3517_devices));

	tam3517_flash_init();
#if defined(CONFIG_USB_MUSB_SOC) || defined(CONFIG_USB_MUSB_SOC_MODULE)
	usb_musb_init();
#endif

#if defined(CONFIG_USB_EHCI_HCD) || defined(CONFIG_USB_EHCI_HCD_MODULE)
	/* Configure GPIO for EHCI port */
	omap_mux_init_gpio(CONFIG_TAM3517_EHCI_RESET_PIN, OMAP_PIN_OUTPUT);
	usb_ehci_init(&ehci_pdata);
#endif

	/*  touchscreen */
	ads7846_dev_init();

        /* MMC init function */
        tam3517_mmc_init(mmc);
	tam3517_mmc2_wifi_init();

        if(gpio_request(CONFIG_TAM3517_POWER_RESET_PIN, "power reset")) {
                printk(KERN_ERR "failed to get power reset\n");
                return;
        }

        omap_mux_init_gpio(CONFIG_TAM3517_POWER_RESET_PIN, OMAP_PIN_OUTPUT);
        gpio_direction_output(CONFIG_TAM3517_POWER_RESET_PIN, 0);
        gpio_free(CONFIG_TAM3517_POWER_RESET_PIN);

}

static void __init tam3517_map_io(void)
{
	omap2_set_globals_343x();
	omap2_map_common_io();
}

MACHINE_START(TAM3517, "TAM3517 EVM")
	.phys_io	= 0x48000000,
	.io_pg_offst	= ((0xd8000000) >> 18) & 0xfffc,
	.boot_params	= 0x80000100,
	.map_io		= tam3517_map_io,
	.init_irq	= tam3517_init_irq,
	.init_machine	= tam3517_init,
	.timer		= &omap_timer,
MACHINE_END
